@@ -21,7 +21,7 @@ from flax.nn import activation
 from flax.nn import initializers
 from flax.nn.activation import (celu, elu, gelu, glu, leaky_relu, log_sigmoid,
                                 log_softmax, relu, sigmoid, soft_sign, softmax,
-                                softplus, swish, tanh)
+                                softplus, swish, silu, tanh)
 from flax.nn.pooling import avg_pool, max_pool
 from .linear import Embedding, conv, conv_transpose, dense, dense_general, embedding
 from .normalization import batch_norm, group_norm, layer_norm
