@@ -18,18 +18,19 @@ from collections.abc import Iterable  # pylint: disable=g-importing-member
 
 import warnings
 
-from .. import jax_utils
-from . import base
-from . import initializers
-from . import stochastic
+from flax import jax_utils
+from flax.nn.activation import softmax
+from flax.nn.base import Collection, Module, collection_from_iterable, iterate_collection
+from flax.nn.initializers import zeros
+from flax.nn.stochastic import make_rng
+from flax.nn.linear import DenseGeneral, default_kernel_init
 from flax import struct
 
 import jax
 from jax import lax
 from jax import random
 import jax.numpy as jnp
-from .linear import default_kernel_init
-from .linear import DenseGeneral
+
 import numpy as onp
 
 
@@ -112,13 +113,13 @@ def dot_product_attention(query,
 
   # normalize the attention weights
   norm_dims = tuple(range(attn_weights.ndim - len(axis), attn_weights.ndim))
-  attn_weights = jax.nn.softmax(attn_weights, axis=norm_dims)
+  attn_weights = softmax(attn_weights, axis=norm_dims)
   attn_weights = attn_weights.astype(dtype)
 
   # apply dropout
   if not deterministic and dropout_rate > 0.:
     if dropout_rng is None:
-      dropout_rng = stochastic.make_rng()
+      dropout_rng = make_rng()
     keep_prob = jax.lax.tie_in(attn_weights, 1.0 - dropout_rate)
     if broadcast_dropout:
       # dropout is broadcast across the batch+head+non-attention dimension
@@ -164,7 +165,7 @@ def scan_in_dim(*args, **kwargs):
   return jax_utils.scan_in_dim(*args, **kwargs)
 
 
-class Cache(base.Collection):
+class Cache(Collection):
   """Collect intermediate activations for efficient autoregressive decoding."""
 
   def initialize_cache(self, shape, dtype=None):
@@ -192,10 +193,10 @@ class Cache(base.Collection):
 
 
 jax.tree_util.register_pytree_node(
-    Cache, base.iterate_collection, base.collection_from_iterable)
+    Cache, iterate_collection, collection_from_iterable)
 
 
-class MultiHeadDotProductAttention(base.Module):
+class MultiHeadDotProductAttention(Module):
   """Multi-head dot-product attention."""
 
   def apply(self,
@@ -218,7 +219,7 @@ class MultiHeadDotProductAttention(base.Module):
             deterministic=False,
             precision=None,
             kernel_init=default_kernel_init,
-            bias_init=initializers.zeros,
+            bias_init=zeros,
             bias=True,
             attention_fn=dot_product_attention):
     """Applies multi-head dot product attention on the input data.
@@ -245,8 +246,9 @@ class MultiHeadDotProductAttention(base.Module):
       causal_mask: boolean specifying whether to apply a causal mask on the
         attention weights. If True, the output at timestep `t` will not depend
         on inputs at timesteps strictly greater than `t`.
-      padding_mask: boolean specifying query tokens that are pad token.
-      key_padding_mask: boolean specifying key-value tokens that are pad token.
+      padding_mask: boolean specifying query tokens that are pad token w/ False.
+      key_padding_mask: boolean specifying key-value tokens that are pad token
+        w/ False.
       segmentation: segment indices for packed inputs_q data.
       key_segmentation: segment indices for packed inputs_kv data.
       cache: an instance of `flax.nn.attention.Cache` used for efficient
@@ -274,6 +276,8 @@ class MultiHeadDotProductAttention(base.Module):
     if inputs_kv is None:
       inputs_kv = inputs_q
 
+    is_self_attention = inputs_kv is inputs_q
+
     if attention_axis is None:
       attention_axis = tuple(range(1, inputs_q.ndim - 1))
 
@@ -332,11 +336,6 @@ class MultiHeadDotProductAttention(base.Module):
                                           value=value)
         cache.store(cache_entry)
 
-        # TODO(levskaya): verify this is still needed in translation decoding.
-        key_padding_mask = jnp.broadcast_to(
-            (jnp.arange(cshape[1]) < cache_entry.i), cshape[:2])
-        key_padding_mask = key_padding_mask.astype(jnp.float32)[..., None]
-
     # create attention masks
     mask_components = []
 
@@ -351,9 +350,20 @@ class MultiHeadDotProductAttention(base.Module):
       else:
         mask_components.append(_make_causal_mask(key, attention_axis))
 
-    if padding_mask is not None:
+    if (padding_mask is not None or key_padding_mask is not None) and not cache:
       if key_padding_mask is None:
-        key_padding_mask = padding_mask
+        if is_self_attention:
+          key_padding_mask = padding_mask
+        else:
+          key_padding_shape = [inputs_kv.shape[dim] for dim in attention_axis]
+          key_padding_mask = jnp.full(key_padding_shape, True)
+      if padding_mask is None:
+        if is_self_attention:
+          padding_mask = key_padding_mask
+        else:
+          padding_shape = [inputs_q.shape[dim] for dim in attention_axis]
+          padding_mask = jnp.full(padding_shape, True)
+
       padding_mask = make_padding_mask(
           padding_mask_query=padding_mask,
           padding_mask_key=key_padding_mask,
@@ -364,6 +374,7 @@ class MultiHeadDotProductAttention(base.Module):
 
     if segmentation is not None:
       if key_segmentation is None:
+        assert is_self_attention
         key_segmentation = segmentation
       segmentation_mask = make_padding_mask(
           padding_mask_query=segmentation,
