@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Initializers for Flax.
 """
 
@@ -25,6 +24,7 @@ from jax.nn.initializers import lecun_uniform
 from jax.nn.initializers import normal
 from jax.nn.initializers import ones
 from jax.nn.initializers import orthogonal
+from jax.nn.initializers import delta_orthogonal
 from jax.nn.initializers import uniform
 from jax.nn.initializers import variance_scaling
 from jax.nn.initializers import xavier_normal
